﻿local dbversion = 7 -- increment this everytime you move a spellid around or fix the db other than adding to it this will reset the mounts db so that errors will be omitted

local L = LibStub("AceLocale-3.0"):GetLocale("Mounted", false)

Mounted = CreateFrame("frame")
local Mounted = Mounted
Mounted:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
Mounted:RegisterEvent("ADDON_LOADED")
function Mounted:Print(...) ChatFrame1:AddMessage(string.join(" ", "|cFF33FF99Mounted|r:", ...)) end

function Mounted:ADDON_LOADED(event, addon)
	local _, class = UnitClass("player")
	if addon:lower() ~= "mounted" then return end
	for k in pairs (self.macros[class]) do
		for name, macro in pairs (self.macros[class][k]) do
			self.macros['GENERAL'][k][name] = macro
		end
	end
	self.locked = {}
	for k in pairs (self.macros['GENERAL']) do
		self.locked[k] = {}
		for name in pairs (self.macros['GENERAL'][k]) do
			self.locked[k][name] = true
		end
	end
	
	Mounted.defaults = {
		['class'] = {
			f = 'Default',
			g = 'Default',
			aq = 'Default',
			cm = 'Default',
		},
		['char'] = {
			floc = 'class',
			gloc = 'class',
			aqloc = 'class',
			cmloc = 'class',
			Module = {},
			mounts = {['g']={['100']={}, ['60']={}, ['40']={}, ['0']={}, ['aq']={},}, ['f']={['310']={}, ['280']={}, ['60']={},}, ['a']={['a']={},},} -- use the a,a classification for spells/things that should be saved but not displayed
		},
		['global'] = {
			['macros'] = self.macros['GENERAL'],
		}
	}
	for module in pairs(self.modules) do
		self.modules[module].LoadModule()
	end
	self.db = LibStub("AceDB-3.0"):New("MountedDB", Mounted.defaults, 'class')
	
	if not self.db.char.dbversion or self.db.char.dbversion < dbversion then
		self:Print(L["DBerror"])
		self.db.char.mounts = {['g']={['100']={}, ['60']={}, ['40']={}, ['0']={}, ['aq']={},}, ['f']={['310']={}, ['280']={}, ['60']={},}, ['a']={['a']={},},}
		self.db.char.dbversion = dbversion
	end
	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
	if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
	SLASH_Mounted1 = "/mounted";
	SLASH_Mounted2 = "/mnt";
	SlashCmdList["Mounted"] = function()
		InterfaceOptionsFrame_OpenToCategory(self.optframe.Mounted)
	end
end

local L_DALARAN
local L_WINTERGRASP

function Mounted:PLAYER_LOGIN()
	L_DALARAN = select(3,GetMapZones(4))
	L_WINTERGRASP = select(10,GetMapZones(4))
	if GetMacroIndexByName("Mounted") == 0 then
		CreateMacro("Mounted", 1, '')
	end
	self:CheckDB()
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	for i=1,4 do
		self:LEARNED_SPELL_IN_TAB(event, i)
	end
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("COMPANION_LEARNED")
	self:RegisterEvent("SPELL_UPDATE_USABLE")
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil
end

function Mounted:LEARNED_SPELL_IN_TAB(event, arg1)
	local _, class = UnitClass("player")
	if arg1 == 1 then
		local _, _, offset, numSpells = GetSpellTabInfo(1);
		for s = offset + 1, offset + numSpells do
			if tostring(string.match(GetSpellLink(GetSpellName(s, BOOKTYPE_SPELL)), "spell:(%d+)")) == '54197' then
				self.ColdWeatherFlying = true
				break
			end
		end
	end
	if self.spellmounts and next(self.spellmounts) then
		local _, _, offset, numSpells = GetSpellTabInfo(arg1);
		local spell
		for s = offset + 1, offset + numSpells do
			spell = tonumber(string.match(GetSpellLink(GetSpellName(s, BOOKTYPE_SPELL)), "spell:(%d+)"))
			if self.spellmounts[spell] then
				local spellname = GetSpellInfo(spell)
				local mounttype, mountspeed = string.match(self.spellmounts[spell], '(.),(.+)')
				self.ReverseLookup[spellname] = spell
				self.db.char.mounts[mounttype][mountspeed][spell] = self.db.char.mounts[mounttype][mountspeed][spell] or 5
				if tonumber(mountspeed) then
					if not self.db.char.mounts[mounttype].maxspeed or tonumber(mountspeed) > tonumber(self.db.char.mounts[mounttype].maxspeed) then
						self.db.char.mounts[mounttype].maxspeed = mountspeed
					end
				end
				self.spellmounts[spell] = nil
			end
		end
	end
	for module in pairs(self.modules) do
		if self.modules[module].LEARNED_SPELL_IN_TAB then
			self.modules[module].LEARNED_SPELL_IN_TAB()
		end
	end
	if self.ColdWeatherFlying and not (self.spellmounts and next(self.spellmounts)) then
		self:UnregisterEvent("LEARNED_SPELL_IN_TAB")
		self:SelectMacro()
		self:SelectMacro()
	end
	self:UpdateOptions()
end

function Mounted:UNIT_SPELLCAST_START(event, arg1, arg2)
	if arg1 == 'player' and self.ReverseLookup[arg2] then
		self:SelectMacro(true)
	end
end

function Mounted:PLAYER_REGEN_DISABLED()
	self:CombatMacro()
end

function Mounted:PLAYER_REGEN_ENABLED()
	self:SelectMacro()
end

function Mounted:SPELL_UPDATE_USABLE()
	for module in pairs(self.modules) do
		if self.modules[module].SPELL_UPDATE_USABLE then
			self.modules[module].SPELL_UPDATE_USABLE()
		end
	end
	self:SelectMacro()
end

function Mounted:COMPANION_LEARNED()
	self:Scan()
end

do
	local macrotype
	function Mounted:CustIsFlyable()
		if IsFlyableArea() then
			local flyable
			local mapContinent = GetCurrentMapContinent()
			local mapZone = GetCurrentMapZone()
			SetMapToCurrentZone()
			local currentcontinent = GetCurrentMapContinent()
			if currentcontinent == 3 or (currentcontinent == 4 and self.ColdWeatherFlying and (GetRealZoneText() ~= L_DALARAN or GetSubZoneText() == L["Krasus' Landing"] or GetSubZoneText() == L["The Underbelly"]) and GetRealZoneText() ~= L_WINTERGRASP) then
				flyable = true
			else
				flyable = false
			end
			SetMapZoom(mapContinent, mapZone)
			return flyable
			end
		return false
	end
	
	function Mounted:SelectMacro(overwrite)
		if not InCombatLockdown() then
			if overwrite then macrotype = nil end
			local newmacrotype
			if self.db.char.mounts['f'].maxspeed and self:CustIsFlyable() and IsOutdoors() then
				newmacrotype = 'AirMacro'
			elseif self.db.char.mounts['g'].maxspeed and IsOutdoors() then
				newmacrotype = 'GroundMacro'
			else
				newmacrotype = 'CombatMacro'
			end
			if macrotype ~= newmacrotype then
				macrotype = newmacrotype
				Mounted[macrotype](Mounted)
			end
		end
	end
	
	function Mounted:CombatMacro()
		macrotype = 'CombatMacro'
		self:ParseMacro(self.db.global.macros.cm[self.db[self.db.char.cmloc].cm])
	end
	
	function Mounted:GetRandomMount(mtype, mspeed)
		if mspeed == 'f' then mspeed = self.db.char.mounts[mtype].maxspeed end
		local mounts = {}
		for k, priority in pairs(self.db.char.mounts[mtype][mspeed]) do
			if priority ~= 0 then
				for i=1, priority do
					mounts[#mounts+1] = k
				end
			end
		end
		if not next(mounts) then mounts[1] = next(self.db.char.mounts[mtype][mspeed]) end -- should resolve edge case where all priorities where set to 0
		local randmount = GetSpellInfo(mounts[math.random(1,#mounts)])
		return randmount..'()'
	end

	function Mounted:GroundMacro()
		if GetRealZoneText() == L["Temple of Ahn'Qiraj"] then
			if next(self.db.char.mounts['g']['aq']) then
				self:ParseMacro(self.db.global.macros.aq[self.db[self.db.char.aqloc].aq])
			else
				self:SelectMacro(true)
			end
		else
			self:ParseMacro(self.db.global.macros.g[self.db[self.db.char.gloc].g])
		end
	end

	function Mounted:AirMacro()
		self:ParseMacro(self.db.global.macros.f[self.db[self.db.char.floc].f])
	end
	
	function Mounted:ParseMacro(macrotext, preview)
		local mounts = {}
		local mountscache = {}
		for mtype, mspeed in string.gmatch(macrotext, "<(%a)(%w+)>") do
			if mtype == 's' then
				mounts[#mounts+1] = GetSpellInfo(mspeed)
			else
				if not mountscache[mtype..mspeed] then
					mountscache[mtype..mspeed] = self:GetRandomMount(mtype, mspeed)
				end
				mounts[#mounts+1] = mountscache[mtype..mspeed]
			end
		end
		macrotext = string.format(string.gsub(macrotext, '<%w+>', '%%s'), unpack(mounts))
		if preview then
			return macrotext
		end
		EditMacro(GetMacroIndexByName("Mounted"), "Mounted", 1, macrotext)
	end
end

Mounted.ReverseLookup = {}
function Mounted:Scan()
	local creatureName, creatureSpellID, mounttype, mountspeed
	for id = 1,GetNumCompanions("MOUNT") do
		creatureSpellID = select(3, GetCompanionInfo("MOUNT", id))
		creatureName = GetSpellInfo(creatureSpellID);
		Mounted.ReverseLookup[creatureName] = creatureSpellID
		local mountpack = self:MountType(creatureSpellID)
		if mountpack[1] then
			for i=1,#mountpack/2 do
				self.db.char.mounts[mountpack[i*2-1]][mountpack[i*2]][creatureSpellID] = self.db.char.mounts[mountpack[i*2-1]][mountpack[i*2]][creatureSpellID] or 5
				if tonumber(mountpack[i*2]) then
					if not self.db.char.mounts[mountpack[i*2-1]].maxspeed or (tonumber(mountpack[i*2]) > tonumber(self.db.char.mounts[mountpack[i*2-1]].maxspeed) and not (#mountpack > 2)) then
						self.db.char.mounts[mountpack[i*2-1]].maxspeed = mountpack[i*2]
					end
				end
			end
		else
			self:Print(creatureName..' with SpellID of '..creatureSpellID..' was not parsable sry!')
		end
	end
	self:UpdateOptions()
	self:SelectMacro()
end

local MountIdsLookup = {}
function Mounted:GetAllids()
	for id = 1,GetNumCompanions("MOUNT") do
		local creatureSpellID = select(3, GetCompanionInfo("MOUNT", id)) 
		MountIdsLookup[creatureSpellID] = true
	end
end

function Mounted:CheckDB()
	self:GetAllids()
	for mtype in pairs(self.db.char.mounts) do
		for mspeed in pairs(self.db.char.mounts[mtype]) do
			if mspeed ~= 'maxspeed' then
				for spellid in pairs(self.db.char.mounts[mtype][mspeed]) do
					if MountIdsLookup[spellid] then
						local mountpack = self:MountType(spellid)
						local match = false
						for i=1,#mountpack/2 do
							if mountpack[i*2-1] == mtype and mountpack[i*2] == mspeed then
								match = true
							end
						end
						if not match then
							self.db.char.mounts[mtype][mspeed][spellid] = nil
						end
					else
						self.db.char.mounts[mtype][mspeed][spellid] = nil
					end
				end
			else
				self.db.char.mounts[mtype][mspeed] = nil
			end
		end
	end
	self:Scan()
	MountIdsLookup = nil
	self.GetAllids = nil
	self.CheckDB = nil
end

